@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,19 +15,24 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/calibr.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/calibr.c,v $
  $Author: bas $
  *
- $Revision: 1.27 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.30 $
+ $Date: 1995/04/22 07:30:15 $
  $State: Beta $
  *
  *      GP calibration routine for processor speed dependent
  *      functions.
  */
 
-static char RCSid[] = "$Id: calibr.c,v 1.27 1995/01/08 14:16:21 bas Beta $";
+static char RCSid[] = "$Id: calibr.c,v 1.30 1995/04/22 07:30:15 bas Beta $";
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/errno.h>
 #include <linux/sched.h>
 #include <asm/system.h>
@@ -39,10 +44,10 @@ static char RCSid[] = "$Id: calibr.c,v 1.27 1995/01/08 14:16:21 bas Beta $";
 
 #undef DEBUG
 
-int
+unsigned
 timestamp( void)
 {
-  int count;
+  unsigned count;
   unsigned long flags;
 
   save_flags( flags);
@@ -58,6 +63,7 @@ int
 timediff( int t0, int t1)
 {
   /*  Calculate difference in usec for timestamp results t0 & t1.
+   *  Note that the maximum timespan allowed is 1/HZ or we'll lose ticks!
    */
   if (t1 < t0) {
     t1 += LATCH;
