@@ -20,19 +20,24 @@
  * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
  * USA.
  *
- * $Source: /home/bas/distr/ftape-2.02/RCS/ecc.c,v $
+ * $Source: /home/bas/distr/ftape-2.03b/RCS/ecc.c,v $
  * $Author: bas $
  *
- * $Revision: 1.30 $
- * $Date: 1995/01/08 14:16:21 $
+ * $Revision: 1.32 $
+ * $Date: 1995/04/22 07:30:15 $
  * $State: Beta $
  *
  *      This file contains the Reed-Solomon error correction code 
  *      for the QIC-40/80 floppy-tape driver for Linux.
  */
 static const char RCSid[] =
-  "$Id: ecc.c,v 1.30 1995/01/08 14:16:21 bas Beta $";
+  "$Id: ecc.c,v 1.32 1995/04/22 07:30:15 bas Beta $";
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <stdio.h>
 #include <sys/errno.h>
 
