@@ -17,18 +17,23 @@
  
  Modified for distribution with ftape source. 21 June 94, SJL.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/fc-10.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/fc-10.c,v $
  $Author: bas $
  *
- $Revision: 1.4 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.7 $
+ $Date: 1995/05/07 13:48:24 $
  $State: Beta $
  *
  *      This file contains code for the CMS FC-10 card.
  */
 
-static char RCSid[] = "$Id: fc-10.c,v 1.4 1995/01/08 14:16:21 bas Beta $";
+static char RCSid[] = "$Id: fc-10.c,v 1.7 1995/05/07 13:48:24 bas Beta $";
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <asm/io.h>
 #include "ftape.h"
 #include "fdc-io.h"
@@ -36,6 +41,16 @@ static char RCSid[] = "$Id: fc-10.c,v 1.4 1995/01/08 14:16:21 bas Beta $";
 
 #ifdef PROBE_FC10
 
+/*  This code will only work if the FC-10 (or FC-20) is set to
+ *  BASE = 0x180, IRQ = 2 and DMA = 3 because I suspect these
+ *  settings to be hard coded in the selection sequence.
+ *  Until someone finds out where and how we'll have to forbid
+ *  other settings.
+ */
+#if (FDC_BASE != 0x180) || (FDC_IRQ != 9) || (FDC_DMA != 3)
+# error - The FC-10 must be set to use i/o 180, irq 2 or 9 and dma 3 !
+#endif
+
 unsigned short inbs_magic[] = {
   0x3, 0x3, 0x0, 0x4, 0x7, 0x2, 0x5, 0x3, 0x1, 0x4,
   0x3, 0x5, 0x2, 0x0, 0x3, 0x7, 0x4, 0x2,
@@ -45,6 +60,7 @@ unsigned short inbs_magic[] = {
 int fc10_enable( void)
 {
    int i;
+   byte x;
 
    /*  Clear state machine ???
     */
@@ -53,11 +69,17 @@ int fc10_enable( void)
    }
    outb( 0x0, FDC_BASE);
 
-   for (i = 0; i < 8; i++) {
-     if (inb( FDC_BASE + i) != 0x13) {
-       return 0;
+   x = inb( FDC_BASE);
+   if (x == 0x13 || x == 0x93) {
+     for (i = 1; i < 8; i++) {
+       if (inb( FDC_BASE + i) != x) {
+         return 0;
+       }
      }
+   } else {
+     return 0;
    }
+   
    outb( 0x8, FDC_BASE);
 
    for (i = 0; i < 8; i++) {
@@ -98,7 +120,7 @@ int fc10_enable( void)
    outb( 0x0c, fdc.dor);        /* release reset */
    outb( 0x2d, fdc.dor);        /* select drive 1 */
 
-   return 1;
+   return (x == 0x93) ? 2 : 1;
 }
 
 #endif /* CMS_FC10_CONTROLLER*/
