@@ -2,7 +2,7 @@
 #define _FDC_IO_H
 
 /*
- * Copyright (C) 1993,1994 Bas Laarhoven.
+ * Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/fdc-io.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/fdc-io.h,v $
  $Author: bas $
  *
- $Revision: 1.35 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.38 $
+ $Date: 1995/05/10 16:09:36 $
  $State: Beta $
  *
  *      This file contains the low level functions
@@ -89,9 +89,8 @@ typedef struct {
   unsigned short fifo;          /* Data register / Fifo on 82077 */
   unsigned short dir;           /* Digital Input Register */
   unsigned short ccr;           /* Configuration Control Register */
-#ifdef MACH2
-  unsigned short dor2;          /* Alternate dor on MACH-2 controller */
-#endif
+  unsigned short dor2;          /* Alternate dor on MACH-2 controller,
+                                   also used with FC-10, meaning unknown */
 } fdc_config_info;
 
 typedef enum {
@@ -165,5 +164,7 @@ extern int fdc_sense_interrupt_status( int *st0, int *current_cylinder);
 extern void fdc_set_data_rate( int rate);
 extern int fdc_release_irq_and_dma( void);
 extern int fdc_init( void);
+extern int fdc_uninit( void);
+extern void fdc_set_write_precomp( int precomp);
 
 #endif
