@@ -2,7 +2,7 @@
 #define _FDC_ISR_H
 
 /*
- * Copyright (C) 1993,1994 Bas Laarhoven.
+ * Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/fdc-isr.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/fdc-isr.h,v $
  $Author: bas $
  *
- $Revision: 1.7 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.8 $
+ $Date: 1995/04/22 07:30:15 $
  $State: Beta $
  *
  *      This file contains the low level functions
