@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-io.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-io.h,v $
  $Author: bas $
  *
- $Revision: 1.33 $
- $Date: 1995/01/15 13:19:55 $
+ $Revision: 1.36 $
+ $Date: 1995/05/06 16:11:53 $
  $State: Beta $
  *
  *      This file contains definitions for the glue part
@@ -47,6 +47,9 @@ typedef struct {
 extern timeout_table timeout;
 extern int qic_std;
 extern int tape_len;
+extern volatile int current_command;
+extern const struct qic117_command_table qic117_cmds[];
+extern int might_be_off_track;
 
 /*
  *      ftape-io.c defined global functions.
