@@ -2,7 +2,7 @@
 #define _FTAPE_READ_H
 
 /*
- * Copyright (C) 1994 Bas Laarhoven.
+ * Copyright (C) 1994-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-read.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-read.h,v $
  $Author: bas $
  *
- $Revision: 1.10 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.13 $
+ $Date: 1995/05/10 16:09:36 $
  $State: Beta $
  *
  *      This file contains the definitions for the read functions
@@ -33,9 +33,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 /*      ftape-read.c defined global vars.
  */
-extern int buf_pos_rd;
-extern int buf_len_rd;
-extern int just_before_eof;
 
 /*      ftape-read.c defined global functions.
  */
