@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-rw.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-rw.c,v $
  $Author: bas $
  *
- $Revision: 1.48 $
- $Date: 1995/01/15 15:56:46 $
+ $Revision: 1.54 $
+ $Date: 1995/05/27 08:55:27 $
  $State: Beta $
  *
  *      This file contains some common code for the segment read and segment
@@ -27,9 +27,14 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-rw.c,v 1.48 1995/01/15 15:56:46 bas Beta bas $";
+"$Id: ftape-rw.c,v 1.54 1995/05/27 08:55:27 bas Beta $";
 
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/string.h>
 #include <linux/errno.h>
 
@@ -51,6 +56,7 @@ volatile enum runner_status_enum runner_status = idle;
 byte deblock_buffer[ (SECTORS_PER_SEGMENT - 3) * SECTOR_SIZE];
 byte scratch_buffer[ (SECTORS_PER_SEGMENT - 3) * SECTOR_SIZE];
 buffer_struct buffer[ NR_FTAPE_BUFFERS];
+int ftape_num_buffers = NR_ITEMS( buffer);
 struct wait_queue *wait_intr = NULL;
 volatile int head;
 volatile int tail;              /* not volatile but need same type as head */
@@ -74,7 +80,7 @@ static int inhibit_correction = 0;
 buffer_struct*
 next_buffer( volatile int* x)
 {
-  if (++*x >= NR_ITEMS( buffer)) {
+  if (++*x >= ftape_num_buffers) {
     *x = 0;
   }
   return &buffer[ *x];
@@ -286,36 +292,50 @@ ftape_read_id( void)
   if (result < 0) {
     TRACE( 1, "fdc_command failed");
   } else {
-    /* set timeout so that the tape will run to logical EOT if
-     * we missed the last sector and there are no INDEX cue pulses.
-     */
-    result = fdc_interrupt_wait( 10 * SECOND);
-    if (result == 0 && fdc_sect != 0) {
-      location.segment = (segments_per_head * fdc_head
-                          + segments_per_cylinder * fdc_cyl
-                          + (fdc_sect - 1) / SECTORS_PER_SEGMENT);
-      location.sector = (fdc_sect - 1) % SECTORS_PER_SEGMENT;
-      location.eot =
-      location.bot = 0;
-      location.known = 1;
-    } else if (result == -ETIME || (result == 0 && fdc_sect == 0)) {
-      result = ftape_report_drive_status( &status);
-      if (result == 0) {
-        if (status & QIC_STATUS_READY) {
-          tape_running = 0;
-          TRACE( 5, "tape has stopped");
-          check_bot_eot( status);
-          if (!location.known) {
+    result = fdc_interrupt_wait( 3 * SECOND);
+    if (result == 0) {
+      if (fdc_sect == 0) {
+        result = ftape_report_drive_status( &status);
+        if (result == 0) {
+          if (status & QIC_STATUS_READY) {
+            tape_running = 0;
+            TRACE( 5, "tape has stopped");
+            check_bot_eot( status);
+            if (!location.known) {
+              result = -EIO;
+            }
+          } else {
+            /*  If read-id failed because of a hard or soft
+             *  error, return an error. Higher level must retry!
+             */
             result = -EIO;
           }
-        } else {
-          TRACE( 1, "weird, drive busy but no data");
+        }
+      } else {
+        location.known = 1;
+        location.segment = (segments_per_head * fdc_head
+                            + segments_per_cylinder * fdc_cyl
+                            + (fdc_sect - 1) / SECTORS_PER_SEGMENT);
+        location.sector = (fdc_sect - 1) % SECTORS_PER_SEGMENT;
+        location.eot =
+        location.bot = 0;
+      }
+    } else if (result == -ETIME) {
+      /*  Didn't find id on tape, must be near end: Wait until stopped.
+       */
+      result = ftape_ready_wait( FOREVER, &status);
+      if (result >= 0) {
+        tape_running = 0;
+        TRACE( 5, "tape has stopped");
+        check_bot_eot( status);
+        if (!location.known) {
           result = -EIO;
         }
       }
     } else {
       /* Interrupted or otherwise failing fdc_interrupt_wait()
        */
+      TRACE( 1, "fdc_interrupt_wait failed :(");
       result = -EIO;
     }
   }
@@ -382,6 +402,7 @@ ftape_dumb_stop( void)
    *  but eventually the drive will become ready (logical eot).
    */
   result = ftape_report_drive_status( &status);
+  location.known = 0;
   do {
     if (result == 0 && status & QIC_STATUS_READY) {
       /* Tape is not running any more.
@@ -392,6 +413,9 @@ ftape_dumb_stop( void)
     } else if (tape_running) {
       /* Tape is (was) still moving.
        */
+#ifdef TESTING
+      ftape_read_id();
+#endif
       result = stop_tape( &status);
     } else {
       /* Tape not yet ready but stopped.
@@ -399,7 +423,9 @@ ftape_dumb_stop( void)
       result = ftape_ready_wait( timeout.pause, &status);
     }
   } while (tape_running);
+#ifndef TESTING
   location.known = 0;
+#endif
   TRACE_EXIT;
   return result;
 }
@@ -454,6 +480,7 @@ fast_seek( int count, int reverse)
      *  we cannot (slow) reverse seek to the last segment.
      *  TO BE IMPLEMENTED.
      */
+    inhibit_correction = 0;
     if (location.known &&
         ((location.bot && !reverse) ||
          (location.eot && reverse))) {
@@ -471,37 +498,70 @@ fast_seek( int count, int reverse)
          *  and overshoot when seeking backwards.
          */
         inhibit_correction = 1;
+        count += 3;             /* best guess */
       }
     }
   } else {
     TRACEx1( 5, "warning: zero or negative count: %d", count);
   }
   if (count > 0) {
+    int i;
+    int nibbles = count > 255 ? 3 : 2;
+
+    if (count > 4095) {
+      TRACE( 4, "skipping clipped at 4095 segment");
+      count = 4095;
+    }
     /* Issue this tape command first. */
     if (!reverse) {
       TRACEx1( 4, "skipping %d segment(s)", count);
-      result = ftape_command( QIC_SKIP_FORWARD);
+      result = ftape_command( nibbles == 3 ?
+                             QIC_SKIP_EXTENDED_FORWARD : QIC_SKIP_FORWARD);
     } else {
       TRACEx1( 4, "backing up %d segment(s)", count);
-      result = ftape_command( QIC_SKIP_REVERSE);
+      result = ftape_command( nibbles == 3 ?
+                             QIC_SKIP_EXTENDED_REVERSE : QIC_SKIP_REVERSE);
     }
-    if (result >= 0) {
-      /* Issue the low nibble of the command. */
-      result = ftape_parameter( 2 + ((count - 1) & 0x0f));
-      if (result >= 0) {
-        /* Issue the high nibble and wait for the command to complete. */
-        result = ftape_parameter( 2 + (((count - 1) >> 4) & 0x0f));
+    if (result < 0) {
+      TRACE( 4, "Skip command failed");
+    } else {
+      --count;                  /* 0 means one gap etc. */
+      for (i = 0 ; i < nibbles ; ++i) {
         if (result >= 0) {
-          result = ftape_ready_wait( timeout.rewind, &status);
-          tape_running = 0;
+          result = ftape_parameter( count & 15);
+          count /= 16;
         }
       }
+      result = ftape_ready_wait( timeout.rewind, &status);
+      if (result >= 0) {
+        tape_running = 0;
+      }
     }
   }
   TRACE_EXIT;
   return result;
 }
 
+static int
+validate( int id)
+{
+  /*  Check to see if position found is off-track as reported once.
+   *  Because all tracks in one direction lie next to each other,
+   *  if off-track the error will be approximately 2 * segments_per_track.
+   */
+  if (location.track == -1) {
+    return 1;                   /* unforseen situation, don't generate error */
+  } else {
+    /*  Use margin of segments_per_track on both sides because ftape
+     *  needs some margin and the error we're looking for is much larger !
+     */
+    int lo = (location.track - 1) * segments_per_track;
+    int hi = (location.track + 2) * segments_per_track;
+
+    return (id >= lo && id < hi);
+  }
+}
+
 static int
 seek_forward( int segment_id)
 {
@@ -514,53 +574,84 @@ seek_forward( int segment_id)
   static int min_count = 8;
   int expected = -1;
   int target = segment_id - margin;
-  int adjust_overshoot = 0;
+  int fast_seeking;
+
+  if (!location.known) {
+    TRACE( 1, "fatal: cannot seek from unknown location");
+    result = -EIO;
+  } else if (!validate( segment_id)) {
+    TRACE( 1, "fatal: head off track (bad hardware?)");
+    ftape_sleep( 1 * SECOND);
+    ftape_failure = 1;
+    result = -EIO;
+  } else {
+    int prev_segment = location.segment;
 
-  if (location.known) {
-    count = target - location.segment - overshoot;
     TRACEx4( 4, "from %d/%d to %d/0 - %d", location.segment,
             location.sector, segment_id, margin);
-    if (count > min_count + (location.bot ? 1 : 0)) {
+    count = target - location.segment - overshoot;
+    fast_seeking = (count > min_count + (location.bot ? 1 : 0));
+    if (fast_seeking) {
       TRACEx1( 5, "fast skipping %d segments", count);
       expected = segment_id - margin;
       fast_seek( count, 0);
-      adjust_overshoot = 1;
     }
     if (!tape_running) {
       logical_forward();
     }
     while (location.segment < segment_id) {
+      /*  This requires at least one sector in a (bad) segment to
+       *  have a valid and readable sector id !
+       *  It looks like this is not guaranteed, so we must try
+       *  to find a way to skip an EMPTY_SEGMENT. !!! FIXME !!!
+       */
       if (ftape_read_id() < 0 || !location.known) {
         location.known = 0;
-        if (!tape_running || ++failures > SECTORS_PER_SEGMENT) {
+        if (!tape_running || ++failures > SECTORS_PER_SEGMENT ||
+            (current->signal & _DONT_BLOCK)) {
           TRACE( 1, "read_id failed completely");
           result = -EIO;
           break;
         } else {
           TRACEx1( 5, "read_id failed, retry (%d)", failures);
         }
-      } else if (adjust_overshoot) {
-        TRACEx2( 4, "ended at %d/%d", location.segment, location.sector);
-        if (!inhibit_correction && location.segment != expected) {
-          TRACEx1( 4, "old overshoot: %d", overshoot);
-          overshoot += location.segment - expected;
-          if (overshoot < 0) {
-            overshoot = 0;
+      } else if (fast_seeking) {
+        TRACEx4( 4, "ended at %d/%d (%d,%d)", location.segment,
+                location.sector, overshoot, inhibit_correction);
+        if (!inhibit_correction &&
+            (location.segment < expected ||
+             location.segment > expected + margin)) {
+          int error = location.segment - expected;
+          TRACEx2( 4, "adjusting overshoot from %d to %d",
+                  overshoot, overshoot + error);
+          overshoot += error;
+          /*  All overshoots have the same direction, so it should
+           *  never become negative, but who knows.
+           */
+          if (overshoot < -5 || overshoot > 10) {
+            if (overshoot < 0) {
+              overshoot = -5;   /* keep sane value */
+            } else {
+              overshoot = 10;   /* keep sane value */
+            }
+            TRACEx1( 4, "clipped overshoot to %d", overshoot);
           }
-          TRACEx1( 4, "new overshoot: %d", overshoot);
         }
-        adjust_overshoot = 0;   /* flag done */
+        fast_seeking = 0;
+      }
+      if (location.known) {
+        if (location.segment > prev_segment + 1) {
+          TRACEx1( 4, "missed segment %d while skipping", prev_segment + 1);
+        }
+        prev_segment = location.segment;
       }
     }
     if (location.segment > segment_id) {
-      TRACE( 5, "skip failed completely");
+      TRACEx2( 4, "failed: skip ended at segment %d/%d",
+              location.segment, location.sector);
       result = -EIO;
     }
-  } else {
-    TRACE( 1, "fatal: cannot seek from unknown location");
-    result = -EIO;
   }
-  inhibit_correction = 0;
   TRACE_EXIT;
   return result;
 }
@@ -571,58 +662,78 @@ skip_reverse( int segment_id, int* pstatus)
   TRACE_FUN( 5, "skip_reverse");
   int result = 0;
   int failures = 0;
-  static int min_rewind = 3;
   static int overshoot = 1;
-  static int margin = 1;        /* fixed: stop this before target */
+  static int min_rewind = 2;    /* 1 + overshoot */
+  static const int margin = 1;  /* stop this before target */
   int expected = 0;
   int count;
-  int min_seek;
+  int short_seek;
   int target = segment_id - margin;
 
-  do {
+  if (location.known && !validate( segment_id)) {
+    TRACE( 1, "fatal: head off track (bad hardware?)");
+    ftape_sleep( 1 * SECOND);
+    ftape_failure = 1;
+    result = -EIO;
+  } else do {
     if (!location.known) {
       TRACE( -1, "warning: location not known");
     }
     TRACEx4( 4, "from %d/%d to %d/0 - %d",
             location.segment, location.sector, segment_id, margin);
-    if (location.segment - min_rewind <= target) {
-      min_seek = 1;
+    /*  min_rewind == 1 + overshoot_when_doing_minimum_rewind
+     *  overshoot  == overshoot_when_doing_larger_rewind
+     *  Initially min_rewind == 1 + overshoot, optimization
+     *  of both values will be done separately.
+     *  overshoot and min_rewind can be negative as both are
+     *  sums of three components:
+     *  any_overshoot == rewind_overshoot - stop_overshoot - start_overshoot
+     */
+    if (location.segment - target - (min_rewind - 1) < 1) {
+      short_seek = 1;
+    } else {
+      count = location.segment - target - overshoot;
+      short_seek = (count < 1);
+    }
+    if (short_seek) {
+      count = 1;                /* do shortest rewind */
       expected = location.segment - min_rewind;
       if (expected / segments_per_track != location.track) {
         expected = location.track * segments_per_track;
       }
-      count = 1;                /* do minimum rewind */
     } else {
-      min_seek = 0;
       expected = target;
-      count = location.segment - target - overshoot;
     }
     fast_seek( count, 1);
     logical_forward();
     result = ftape_read_id();
     if (result == 0 && location.known) {
-      TRACEx2( 4, "ended at %d/%d", location.segment, location.sector);
+      TRACEx5( 4, "ended at %d/%d (%d,%d,%d)", location.segment,
+              location.sector, min_rewind, overshoot, inhibit_correction);
       if (!inhibit_correction &&
           (location.segment < expected ||
            location.segment > expected + margin)) {
-        if (min_seek) {
-          TRACE( 4, "adjusting min_rewind");
-          TRACEx1( 4, "old value = %d", min_rewind);
-          min_rewind += expected - location.segment;
-          if (min_rewind < 1) {
-            min_rewind = 1;     /* keep sane values */
+        int error = expected - location.segment;
+        if (short_seek) {
+          TRACEx2( 4, "adjusting min_rewind from %d to %d",
+                  min_rewind, min_rewind + error);
+          min_rewind += error;
+          if (min_rewind < -5) { /* is this right ? FIXME ! */
+            min_rewind = -5;    /* keep sane value */
+            TRACEx1( 4, "clipped min_rewind to %d", min_rewind);
           }
-          TRACEx1( 4, "new value = %d", min_rewind);
         } else {
-          TRACE( 4, "adjusting overshoot");
-          TRACEx1( 4, "old overshoot %d", overshoot);
-          overshoot += target - location.segment;
-          if (overshoot < 0) {
-            overshoot = 0;
-          } else if (overshoot > min_rewind) {
-            overshoot = min_rewind;
+          TRACEx2( 4, "adjusting overshoot from %d to %d",
+                  overshoot, overshoot + error);
+          overshoot += error;
+          if (overshoot < -5 || overshoot > 10) {
+            if (overshoot < 0) {
+              overshoot = -5;   /* keep sane value */
+            } else {
+              overshoot = 10;   /* keep sane value */
+            }
+            TRACEx1( 4, "clipped overshoot to %d", overshoot);
           }
-          TRACEx1( 4, "new overshoot %d", overshoot);
         }
       }
     } else {
@@ -640,8 +751,11 @@ skip_reverse( int segment_id, int* pstatus)
         break;
       }
     }
-  } while (location.segment > segment_id);
-  inhibit_correction = 0;
+  } while (location.segment > segment_id &&
+           (current->signal & _DONT_BLOCK) == 0);
+  if (location.known) {
+    TRACEx2( 4, "current location: %d/%d", location.segment, location.sector);
+  }
   TRACE_EXIT;
   return result;
 }
@@ -761,6 +875,7 @@ ftape_start_tape( int segment_id, int sector_offset)
    * and being able to access the first sector.
    */
   static int start_offset = 1;
+  int retry = 0;
 
   /* If sector_offset > 0, seek into wanted segment instead of
    * into previous.
@@ -781,7 +896,8 @@ ftape_start_tape( int segment_id, int sector_offset)
     }
   }
   last_segment = segment_id;
-  if (location.track != track) {
+  if (location.track != track || (might_be_off_track &&
+                                  buffer[ head].retry == 0)) {
     /* current track unknown or not equal to destination
      */
     ftape_ready_wait( timeout.seek, &status);
@@ -826,11 +942,11 @@ ftape_start_tape( int segment_id, int sector_offset)
     if (!location.known) {
       TRACE( -1, "panic: location not known");
       result = -EIO;
-#if 0
-      break;
-#else
-      continue;
-#endif
+      if ((current->signal & _DONT_BLOCK) || ftape_failure) {
+        break;
+      } else {
+        continue;
+      }
     }
     TRACEx2( 4, "current segment: %d/%d", location.segment, location.sector);
     /*  We're on the right track somewhere before the wanted segment.
@@ -850,7 +966,9 @@ ftape_start_tape( int segment_id, int sector_offset)
         location.segment != segment_id - (sector_offset > 0 ? 0 : 1)) {
       result = -EIO;
     }
-  } while (result < 0 && (current->signal & ~current->blocked) == 0);
+  } while (result < 0 && !ftape_failure &&
+           (current->signal & _DONT_BLOCK) == 0 &&
+           ++retry <= 5);
   if (result < 0) {
     TRACE( 1, "failed to reposition");
   }
