@@ -2,7 +2,7 @@
 #define _FTAPE_RW_H
 
 /*
- * Copyright (C) 1993,1994 Bas Laarhoven.
+ * Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-rw.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-rw.h,v $
  $Author: bas $
  *
- $Revision: 1.32 $
- $Date: 1995/01/10 19:48:07 $
+ $Revision: 1.33 $
+ $Date: 1995/04/22 07:30:15 $
  $State: Beta $
  *
  *      This file contains the definitions for the read and write
@@ -140,6 +140,7 @@ typedef enum {
 extern int tracing;
 extern byte trace_id;
 extern buffer_struct buffer[ NR_FTAPE_BUFFERS];
+extern int ftape_num_buffers;
 extern location_record location;
 extern volatile ftape_fast_start_struct ftape_fast_start;
 extern byte deblock_buffer[ (SECTORS_PER_SEGMENT - 3) * SECTOR_SIZE];
