@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-write.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-write.c,v $
  $Author: bas $
  *
- $Revision: 1.21 $
- $Date: 1995/01/15 15:56:46 $
+ $Revision: 1.26 $
+ $Date: 1995/05/27 08:55:27 $
  $State: Beta $
  *
  *      This file contains the writing code
@@ -27,11 +27,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-write.c,v 1.21 1995/01/15 15:56:46 bas Beta bas $";
+"$Id: ftape-write.c,v 1.26 1995/05/27 08:55:27 bas Beta $";
 
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/string.h>
 #include <linux/errno.h>
+#include <linux/mm.h>
 #include <asm/segment.h>
 
 #include "ftape.h"
@@ -63,7 +69,7 @@ ftape_zap_write_buffers( void)
 {
   int i;
 
-  for (i = 0; i < NR_ITEMS( buffer); ++i) {
+  for (i = 0; i < ftape_num_buffers; ++i) {
     buffer[ i].status = done;
   }
   need_flush = 0;
@@ -150,8 +156,8 @@ loop_until_writes_done( void)
    */
   while (ftape_state == writing && buffer[ head].status != done) {
     TRACEx2( 7, "tail: %d, head: %d", tail, head);
-    for (i = 0; i < NR_ITEMS( buffer); ++i) {
-      TRACEx3( 7 /* 8 */, "buffer[ %d] segment_id: %d, status: %d",
+    for (i = 0; i < ftape_num_buffers; ++i) {
+      TRACEx3( 8, "buffer[ %d] segment_id: %d, status: %d",
               i, buffer[ i].segment_id, buffer[ i].status);
     }
     result = fdc_interrupt_wait( 5 * SECOND);
@@ -163,12 +169,16 @@ loop_until_writes_done( void)
     if (buffer[ head].status == error) {
       /* Allow escape from loop when signaled !
        */
-      if (current->signal & ~_BLOCKABLE) {
+      if (current->signal & _DONT_BLOCK) {
         TRACE( 2, "interrupted by signal");
         TRACE_EXIT;
         result = -EINTR;          /* is this the right return value ? */
         break;
       }
+      if (buffer[ head].hard_error_map != 0) {
+        /*  Implement hard write error recovery here
+         */
+      }
       buffer[ head].status = waiting; /* retry this one */
       if (runner_status == aborting) {
         ftape_dumb_stop();
@@ -222,6 +232,8 @@ write_segment( unsigned segment_id, byte* address, int flushing)
       TRACEx1( 1, "warning: %d hard error(s) in written segment",
               count_ones( buffer[ tail].hard_error_map));
       TRACEx1( 4, "hard_error_map = 0x%08lx", buffer[ tail].hard_error_map);
+      /*  Implement hard write error recovery here
+       */
     }
   } else if (buffer[ tail].status == done) {
     history.defects += count_ones( buffer[ tail].hard_error_map);
@@ -331,7 +343,7 @@ _write_segment( unsigned int segment_id, byte* buffer, int flush)
     }
     /* Allow escape from loop when signaled !
      */
-    if (current->signal & ~_BLOCKABLE) {
+    if (current->signal & _DONT_BLOCK) {
       TRACE( 2, "interrupted by signal");
       TRACE_EXIT;
       result = -EINTR;          /* is this the right return value ? */
@@ -447,7 +459,10 @@ ftape_update_header_segments( byte* buffer, int update)
   if (ftape_state == writing) {
     result = loop_until_writes_done();
   }
-  if (result >= 0) {
+  if (read_only) {
+    result = 0;                 /* exit and fake success */
+    TRACE( 4, "Tape set read-only: no update");
+  } else if (result >= 0) {
     result = ftape_abort_operation();
     if (result >= 0) {
       if (buffer == NULL) {
@@ -489,8 +504,6 @@ ftape_flush_buffers( void)
   int result;
   int pad_count;
   int data_remaining;
-  int total_remaining;
-  unsigned nr_sectors;
   static int active = 0;
 
   if (active) {
@@ -506,7 +519,6 @@ ftape_flush_buffers( void)
     TRACE_EXIT;
     return 0;
   }
-  total_remaining =
   data_remaining = buf_pos_wr;
   buf_pos_wr = 0;               /* prevent further writes if this fails */
   TRACE( 5, "flushing write buffers");
@@ -520,14 +532,14 @@ ftape_flush_buffers( void)
    *    If there is any data not written to tape yet, append zero's
    *    up to the end of the sector. Then write the segment(s) to tape.
    */
-  TRACEi( 7, "flush, remainder in buffer:", total_remaining);
-  nr_sectors = (total_remaining + SECTOR_SIZE - 1) / SECTOR_SIZE;
-  TRACEi( 7, "flush, remaining sectors:", nr_sectors);
-  if (nr_sectors > 0) {
+  if (data_remaining > 0) {
+    int written;
+
     do {
-      pad_count = sizeof( deblock_buffer) - total_remaining;
+      TRACEi( 4, "remaining in buffer:", data_remaining);
+      pad_count = sizeof( deblock_buffer) - data_remaining;
       TRACEi( 7, "flush, padding count:", pad_count);
-      memset( deblock_buffer + total_remaining, 0, pad_count); /* pad buffer */
+      memset( deblock_buffer + data_remaining, 0, pad_count); /* pad buffer */
       result = _write_segment( ftape_seg_pos, deblock_buffer, 1);
       if (result < 0) {
         if (result != -ENOSPC) {
@@ -536,35 +548,45 @@ ftape_flush_buffers( void)
         active = 0;
         TRACE_EXIT;
         return result;
-      } else {
-        clear_eof_mark_if_set( ftape_seg_pos, result);
       }
-      TRACEi( 7, "flush, moved out buffer:", result);
-      if (result < data_remaining) { /* need next segment for more data */
-        total_remaining = sizeof( deblock_buffer) - result;
-        if (result > 0) {       /* move remainder to buffer beginning */
-          memmove( deblock_buffer, deblock_buffer + result, total_remaining);
+      written = result;
+      clear_eof_mark_if_set( ftape_seg_pos, written);
+      TRACEi( 7, "flush, moved out buffer:", written);
+      if (written > 0) {
+        data_remaining -= written;
+        if (data_remaining > 0) {
+          /*  Need another segment for remaining data, move the remainder
+           *  to the beginning of the buffer
+           */
+          memmove( deblock_buffer, deblock_buffer + written, data_remaining);
         }
-      } else {
-        total_remaining = 0;        /* no need to write more, reset buffer */
       }
-      data_remaining -= result;
       ++ftape_seg_pos;
     } while (data_remaining > 0);
-    TRACEi( 7, "netto pad-count:", -data_remaining);
-    TRACEi( 7, "result:", result);
-    if (data_remaining == 0) {  /* first sector next segment */
-      ftape_weof( 2, ftape_seg_pos, 1);
-      ftape_seg_pos -= 1;
-    } else {                    /* after data previous segment */
+    /*  Data written to last segment == data_remaining + written
+     *  value is in range [1..29K].
+     */
+    TRACEx2( 4, "last write: %d, netto pad-count: %d",
+            data_remaining + written, -data_remaining);
+    if (-1024 < data_remaining && data_remaining <= 0) {
+      /*  Last sector of segment was used for data, so put eof mark
+       *  in next segment and position at second file mark.
+       */
+      if (ftape_weof( 2, ftape_seg_pos, 1) >= 0) {
+        ++ftape_seg_pos;        /* position between file marks */
+      }
+    } else {
+      /*  Put eof mark in previous segment after data and position
+       *  at second file mark.
+       */
       ftape_weof( 2, ftape_seg_pos - 1, 1 +
                  ((SECTOR_SIZE - 1 + result + data_remaining) / SECTOR_SIZE));
-      ftape_seg_pos -= 2;
     }
   } else {
     TRACE( 7, "deblock_buffer empty");
-    ftape_weof( 2, ftape_seg_pos, 1);
-    ftape_seg_pos -= 1;
+    if (ftape_weof( 2, ftape_seg_pos, 1) >= 0) {
+      ++ftape_seg_pos;          /* position between file marks */
+    }
     start_writing( WRITE_MULTI);
   }
   TRACE( 7, "waiting");
@@ -626,17 +648,19 @@ _ftape_write( char* buff, int req_len)
     TRACEi( 7, "remaining req_len:", req_len);
     TRACEi( 7, "          buf_pos:", buf_pos_wr);
     cnt = (req_len < space_left) ? req_len : space_left;
-    result = verify_area( VERIFY_READ, buff, cnt);
-    if (result) {
-      TRACE( 1, "verify_area failed");
-      last_write_failed = 1;
-      TRACE_EXIT;
-      return result;
+    if (cnt > 0) {
+      result = verify_area( VERIFY_READ, buff, cnt);
+      if (result) {
+        TRACE( 1, "verify_area failed");
+        last_write_failed = 1;
+        TRACE_EXIT;
+        return result;
+      }
+      memcpy_fromfs( deblock_buffer + buf_pos_wr, buff, cnt);
+      buff += cnt;
+      req_len -= cnt;
+      buf_pos_wr += cnt;
     }
-    memcpy_fromfs( deblock_buffer + buf_pos_wr, buff, cnt);
-    buff += cnt;
-    req_len -= cnt;
-    buf_pos_wr += cnt;
     TRACEi( 7, "moved into blocking buffer:", cnt);
     while (buf_pos_wr >= sizeof( deblock_buffer)) {
       /*  If this is the last buffer to be written, let flush handle it.
@@ -672,7 +696,7 @@ _ftape_write( char* buff, int req_len)
       ++ftape_seg_pos;
       /* Allow us to escape from this loop with a signal !
        */
-      if (current->signal & ~_BLOCKABLE) {
+      if (current->signal & _DONT_BLOCK) {
         TRACE( 2, "interrupted by signal");
         last_write_failed = 1;
         TRACE_EXIT;
