@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/kernel-interface.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/kernel-interface.c,v $
  $Author: bas $
  *
- $Revision: 1.43 $
- $Date: 1995/01/15 13:19:55 $
+ $Revision: 1.55 $
+ $Date: 1995/05/27 12:10:05 $
  $State: Beta $
  *
  *      This file contains the code that interfaces the kernel
@@ -27,16 +27,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: kernel-interface.c,v 1.43 1995/01/15 13:19:55 bas Beta $";
-
+"$Id: kernel-interface.c,v 1.55 1995/05/27 12:10:05 bas Beta $";
 
+#include "kernel-version.h"
+#include <linux/config.h>
+#include <linux/module.h>
 #include <linux/errno.h>
 #include <linux/fs.h>
 #include <asm/segment.h>
 #include <linux/kernel.h>
 #include <linux/signal.h>
 #include <linux/major.h>
-#include <linux/module.h>
 
 #include "ftape.h"
 #include "kernel-interface.h"
@@ -45,13 +46,12 @@ static char RCSid[] =
 #include "ftape-io.h"
 #include "ftape-ctl.h"
 #include "ftape-rw.h"
-#include "fdc-isr.h"
-#include "kernel-version.h"
+
 
 /*      Global vars.
  */
 char *tape_buffer = NULL;
-char kernel_version[] = KERNEL_VERSION;
+char kernel_version[] = UTS_RELEASE;
 
 /*      Local vars.
  */
@@ -89,21 +89,15 @@ static struct file_operations ftape_cdev = {
   NULL,                         /* fsync */
 };
 
-#define SIG_BLOCK_ALL   (0xffffffff)
-
 /*      Called by modules package when installing the driver
  */
 int
 init_module( void) {
   TRACE_FUN( 5, "init_module");
 
-  printk( "ftape v2.02 15/01/95 (c) 1993-1995 Bas Laarhoven (bas@vimec.nl)"
+  printk( "ftape v2.03b 27/05/95 (c) 1993-1995 Bas Laarhoven (bas@vimec.nl)"
          "\n QIC-117 driver for QIC-40/80/3010/3020 tape drives\n"
-#if 0
-         "====== this is not an official release, don't distribute or\n"
-         "       use it unless I specifically asked you so ! Bas. ======\n"
-#endif
-         );
+         " Compiled for kernel version %s\n", kernel_version);
   TRACE( 3, "installing QIC-117 ftape driver...");
   if (register_chrdev( QIC117_TAPE_MAJOR, "ftape", &ftape_cdev)) {
     TRACE( 1, "register_chrdev failed");
@@ -121,8 +115,10 @@ init_module( void) {
   ftape_failure = 1;            /* inhibit any operation but open */
   udelay_calibrate();           /* must be before fdc_wait_calibrate ! */
   fdc_wait_calibrate();
-  ftape_new_cartridge();        /* init some tape related variables */
   TRACE_EXIT;
+#if KERNEL_VERSION >= 1001085
+  register_symtab(0);           /* remove global ftape symbols */
+#endif
   return 0;
 }
 
@@ -142,7 +138,7 @@ cleanup_module( void) {
 
 /*      Open ftape device
  */
-int
+static int
 ftape_open( struct inode* ino, struct file* filep)
 {
   TRACE_FUN( 4, "ftape_open");
@@ -162,10 +158,15 @@ ftape_open( struct inode* ino, struct file* filep)
     TRACE_EXIT;
     return -ENXIO;
   }
+  if (ftape_unit == -1 || FTAPE_UNIT != (MINOR( ino->i_rdev) & 3)) {
+    /*  Other selection than last time
+     */
+    ftape_init_driver();
+  }
   ftape_unit = MINOR( ino->i_rdev);
   ftape_failure = 0;            /* allow tape operations */
   old_sigmask = current->blocked;
-  current->blocked = SIG_BLOCK_ALL;
+  current->blocked = _BLOCK_ALL;
   result = _ftape_open();
   if (result < 0) {
     TRACE( 1, "_ftape_open failed");
@@ -175,7 +176,10 @@ ftape_open( struct inode* ino, struct file* filep)
     return result;
   } else {
     busy_flag = 1;
-    current->blocked = old_sigmask | _S(SIGPIPE);
+    /*  Mask signals that will disturb proper operation of the
+     *  program that is calling.
+     */
+    current->blocked = old_sigmask | _DO_BLOCK;
     TRACE_EXIT;
     return 0;
   }
@@ -183,7 +187,7 @@ ftape_open( struct inode* ino, struct file* filep)
 
 /*      Close ftape device
  */
-void
+static void
 ftape_close( struct inode* ino, struct file* filep)
 {
   TRACE_FUN( 4, "ftape_close");
@@ -194,7 +198,7 @@ ftape_close( struct inode* ino, struct file* filep)
     TRACE_EXIT;
     return;                     /* keep busy_flag !(?) */
   }
-  current->blocked = SIG_BLOCK_ALL;
+  current->blocked = _BLOCK_ALL;
   result = _ftape_close();
   if (result < 0) {
     TRACE( 1, "_ftape_close failed");
@@ -208,7 +212,7 @@ ftape_close( struct inode* ino, struct file* filep)
 
 /*      Ioctl for ftape device
  */
-int
+static int
 ftape_ioctl( struct inode* ino, struct file* filep, 
             unsigned int command, unsigned long arg)
 {
@@ -222,7 +226,7 @@ ftape_ioctl( struct inode* ino, struct file* filep,
     return -EIO;
   }
   old_sigmask = current->blocked; /* save mask */
-  current->blocked = SIG_BLOCK_ALL;
+  current->blocked = _BLOCK_ALL;
   /* This will work as long as sizeof( void*) == sizeof( long)
    */
   result = _ftape_ioctl( command, (void*) arg);
@@ -233,7 +237,7 @@ ftape_ioctl( struct inode* ino, struct file* filep,
 
 /*      Read from tape device
  */
-int
+static int
 ftape_read( struct inode* ino, struct file* fp, char* buff, int req_len)
 {
   TRACE_FUN( 5, "ftape_read");
@@ -247,7 +251,7 @@ ftape_read( struct inode* ino, struct file* fp, char* buff, int req_len)
     return -EIO;
   }
   old_sigmask = current->blocked; /* save mask */
-  current->blocked = SIG_BLOCK_ALL;
+  current->blocked = _BLOCK_ALL;
   result = _ftape_read( buff, req_len);
   TRACEi( 7, "return with count:", result);
   current->blocked = old_sigmask; /* restore mask */
@@ -257,10 +261,10 @@ ftape_read( struct inode* ino, struct file* fp, char* buff, int req_len)
 
 /*      Write to tape device
  */
-int
+static int
 ftape_write( struct inode* ino, struct file* fp, char* buff, int req_len)
 {
-  TRACE_FUN( 5, "ftape_write");
+  TRACE_FUN( 8, "ftape_write");
   int result = -EIO;
   int old_sigmask;
 
@@ -271,7 +275,7 @@ ftape_write( struct inode* ino, struct file* fp, char* buff, int req_len)
     return -EIO;
   }
   old_sigmask = current->blocked; /* save mask */
-  current->blocked = SIG_BLOCK_ALL;
+  current->blocked = _BLOCK_ALL;
   result = _ftape_write( buff, req_len);
   TRACEi( 7, "return with count:", result);
   current->blocked = old_sigmask; /* restore mask */
@@ -281,7 +285,7 @@ ftape_write( struct inode* ino, struct file* fp, char* buff, int req_len)
 
 /*      Seek tape device - not implemented !
  */
-int
+static int
 ftape_lseek( struct inode* ino, struct file* filep, off_t offset, int origin)
 {
   return -ESPIPE;
