@@ -2,7 +2,7 @@
 #define _VENDORS_H
 
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/vendors.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/vendors.h,v $
  $Author: bas $
  *
- $Revision: 1.29 $
- $Date: 1995/01/15 13:19:55 $
+ $Revision: 1.34 $
+ $Date: 1995/05/27 08:54:21 $
  $State: Beta $
  *
  *      This file contains the supported drive types
@@ -66,28 +66,31 @@ typedef struct {
 
 #define QIC117_VENDORS { \
 /* see _vendor_struct */ \
-  { 0x00000,   0, wake_up_colorado,  "Colorado DJ-10 (old)" }, \
+  { 0x00000,  82, wake_up_colorado,  "Colorado DJ-10 (old)" }, \
   { 0x00047,  90, wake_up_colorado,  "Colorado DJ-10/DJ-20" }, \
-  { 0x011c4,   0, wake_up_colorado,  "Colorado DJ-10/DJ-20 (new)" }, \
-  { 0x00005,   0, wake_up_mountain,  "Archive 5580i" }, \
-  { 0x10005,  50, wake_up_insight,   "Insight 80 Mb" }, \
-  { 0x00140,   0, wake_up_mountain,  "Identity IDTBU120E, Escom ?" }, \
-  { 0x00146,  72, wake_up_mountain,  "Escom/Archive 31250Q" }, \
+  { 0x011c3,  90, wake_up_colorado,  "Colorado 1400" }, \
+  { 0x011c4,  84, wake_up_colorado,  "Colorado DJ-10/DJ-20 (new)" }, \
+  { 0x011c5,  84, wake_up_colorado,  "HP Colorado T-1000" }, \
+  { 0x00005,  45, wake_up_mountain,  "Archive 5580i" }, \
+  { 0x10005,  50, wake_up_insight,   "Insight 80 Mb, Irwin 80SX" }, \
+  { 0x00140,  74, wake_up_mountain,  "Archive S.Hornet [Identity/Escom]" }, \
+  { 0x00146,  72, wake_up_mountain,  "Archive 31250Q [Escom]" }, \
   { 0x0014a, 100, wake_up_mountain,  "Archive XL9250i [Conner/Escom]" }, \
-  { 0x0014c,   0, wake_up_mountain,  "Conner C250MQT" }, \
+  { 0x0014c,  98, wake_up_mountain,  "Conner C250MQT" }, \
   { 0x0014e,  80, wake_up_mountain,  "Conner C250MQ" }, \
   { 0x00150,  80, wake_up_mountain,  "Conner TSM420R" }, \
   { 0x00152,  80, wake_up_mountain,  "Conner TSM850R" }, \
+  { 0x00156,  80, wake_up_mountain,  "Conner TSM850R (new)" }, \
   { 0x00180,   0, wake_up_mountain,  "Summit SE 150" }, \
   { 0x00181,  85, wake_up_mountain,  "Summit SE 250, Mountain FS8000" }, \
   { 0x001c1,  82, no_wake_up,        "Wangtek 3040F" }, \
-  { 0x001c8,   0, no_wake_up,        "Wangtek 3080F" }, \
-  { 0x001c8,   0, wake_up_colorado,  "Wangtek 3080F" }, \
-  { 0x001ca,   0, no_wake_up,        "Wangtek 3080F" }, \
+  { 0x001c8,  64, no_wake_up,        "Wangtek 3080F" }, \
+  { 0x001c8,  64, wake_up_colorado,  "Wangtek 3080F" }, \
+  { 0x001ca,  67, no_wake_up,        "Wangtek 3080F (new)" }, \
   { 0x00380,   0, wake_up_colorado,  "Exabyte EXB-1500" }, \
   { 0x08880,  64, no_wake_up,        "Iomega 250" }, \
   { 0x08880,  64, wake_up_colorado,  "Iomega 250" }, \
-  { 0x00021,   0, no_wake_up,        "AIWA CT-803" }, \
+  { 0x00021,  70, no_wake_up,        "AIWA CT-803" }, \
   { 0x00021,   0, wake_up_mountain,  "COREtape QIC80" }, \
 }
 
